"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)
from ..model_utils import OpenApiModel
from agilicus_api.exceptions import ApiAttributeError



class AuthAudits(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @property
    def user_id(self):
       return self.get("user_id")

    @user_id.setter
    def user_id(self, new_value):
       self.user_id = new_value

    @property
    def upstream_user_id(self):
       return self.get("upstream_user_id")

    @upstream_user_id.setter
    def upstream_user_id(self, new_value):
       self.upstream_user_id = new_value

    @property
    def org_id(self):
       return self.get("org_id")

    @org_id.setter
    def org_id(self, new_value):
       self.org_id = new_value

    @property
    def org_name(self):
       return self.get("org_name")

    @org_name.setter
    def org_name(self, new_value):
       self.org_name = new_value

    @property
    def time(self):
       return self.get("time")

    @time.setter
    def time(self, new_value):
       self.time = new_value

    @property
    def event(self):
       return self.get("event")

    @event.setter
    def event(self, new_value):
       self.event = new_value

    @property
    def event_name(self):
       return self.get("event_name")

    @event_name.setter
    def event_name(self, new_value):
       self.event_name = new_value

    @property
    def source_ip(self):
       return self.get("source_ip")

    @source_ip.setter
    def source_ip(self, new_value):
       self.source_ip = new_value

    @property
    def token_id(self):
       return self.get("token_id")

    @token_id.setter
    def token_id(self, new_value):
       self.token_id = new_value

    @property
    def trace_id(self):
       return self.get("trace_id")

    @trace_id.setter
    def trace_id(self, new_value):
       self.trace_id = new_value

    @property
    def session(self):
       return self.get("session")

    @session.setter
    def session(self, new_value):
       self.session = new_value

    @property
    def issuer(self):
       return self.get("issuer")

    @issuer.setter
    def issuer(self, new_value):
       self.issuer = new_value

    @property
    def client_id(self):
       return self.get("client_id")

    @client_id.setter
    def client_id(self, new_value):
       self.client_id = new_value

    @property
    def application_name(self):
       return self.get("application_name")

    @application_name.setter
    def application_name(self, new_value):
       self.application_name = new_value

    @property
    def login_org_id(self):
       return self.get("login_org_id")

    @login_org_id.setter
    def login_org_id(self, new_value):
       self.login_org_id = new_value

    @property
    def login_org_name(self):
       return self.get("login_org_name")

    @login_org_name.setter
    def login_org_name(self, new_value):
       self.login_org_name = new_value

    @property
    def upstream_idp(self):
       return self.get("upstream_idp")

    @upstream_idp.setter
    def upstream_idp(self, new_value):
       self.upstream_idp = new_value

    @property
    def stage(self):
       return self.get("stage")

    @stage.setter
    def stage(self, new_value):
       self.stage = new_value

    @property
    def result(self):
       return self.get("result")

    @result.setter
    def result(self, new_value):
       self.result = new_value

    @property
    def user_agent(self):
       return self.get("user_agent")

    @user_agent.setter
    def user_agent(self, new_value):
       self.user_agent = new_value

    @property
    def request_id(self):
       return self.get("request_id")

    @request_id.setter
    def request_id(self, new_value):
       self.request_id = new_value

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            'user_id': (str,),  # noqa: E501
            'upstream_user_id': (str,),  # noqa: E501
            'org_id': (str,),  # noqa: E501
            'org_name': (str,),  # noqa: E501
            'time': (datetime,),  # noqa: E501
            'event': (str,),  # noqa: E501
            'event_name': (str,),  # noqa: E501
            'source_ip': (str,),  # noqa: E501
            'token_id': (str,),  # noqa: E501
            'trace_id': (str,),  # noqa: E501
            'session': (str,),  # noqa: E501
            'issuer': (str,),  # noqa: E501
            'client_id': (str,),  # noqa: E501
            'application_name': (str,),  # noqa: E501
            'login_org_id': (str,),  # noqa: E501
            'login_org_name': (str,),  # noqa: E501
            'upstream_idp': (str,),  # noqa: E501
            'stage': (str,),  # noqa: E501
            'result': (str,),  # noqa: E501
            'user_agent': (str,),  # noqa: E501
            'request_id': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None



    attribute_map = {
        'user_id': 'user_id',  # noqa: E501
        'upstream_user_id': 'upstream_user_id',  # noqa: E501
        'org_id': 'org_id',  # noqa: E501
        'org_name': 'org_name',  # noqa: E501
        'time': 'time',  # noqa: E501
        'event': 'event',  # noqa: E501
        'event_name': 'event_name',  # noqa: E501
        'source_ip': 'source_ip',  # noqa: E501
        'token_id': 'token_id',  # noqa: E501
        'trace_id': 'trace_id',  # noqa: E501
        'session': 'session',  # noqa: E501
        'issuer': 'issuer',  # noqa: E501
        'client_id': 'client_id',  # noqa: E501
        'application_name': 'application_name',  # noqa: E501
        'login_org_id': 'login_org_id',  # noqa: E501
        'login_org_name': 'login_org_name',  # noqa: E501
        'upstream_idp': 'upstream_idp',  # noqa: E501
        'stage': 'stage',  # noqa: E501
        'result': 'result',  # noqa: E501
        'user_agent': 'user_agent',  # noqa: E501
        'request_id': 'request_id',  # noqa: E501
    }

    read_only_vars = {
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """AuthAudits - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            user_id (str): The system-local id of the user performing the action.. [optional]  # noqa: E501
            upstream_user_id (str): The id of the user in the upstream system, if available.. [optional]  # noqa: E501
            org_id (str): The id of the organisation of the issuer against which the user is authenticating. . [optional]  # noqa: E501
            org_name (str): The name of the organisation of the issuer against which the user is authenticating. . [optional]  # noqa: E501
            time (datetime): the time at which the record was generated.. [optional]  # noqa: E501
            event (str): The event which generated the record. The meaning of the event depends on the stage where it occured. Deprecated: this has been replaced with Result. . [optional]  # noqa: E501
            event_name (str): The event name which generated the record. The following events are supported:    \"Authentication Request\":  An OIDC 3-way handshake authentication request flow (RFC6749)    \"Identity Provider Authentication\": Authentication with an upstream identity provider.    \"Identity Provider Callback\": A callback received during the OIDC 3-way handshake that involves     communicating with the upstream identity provider.    \"Assertion Request\":  Authorization associated with a service account utilizing     an authentication document.    \"Policy Evaluation\": As part of authentication and authorization, policy is being evaluated.    \"Token\": Creation of an Access Token for access to Agilicus resources.    \"MFA Challenge\": A Multifactor method is required and this is communicating to the user     to invoke their MFA method. . [optional]  # noqa: E501
            source_ip (str): The IP address of the host initiating the action. [optional]  # noqa: E501
            token_id (str): The id of the token issued or reissued as part of the authentication.. [optional]  # noqa: E501
            trace_id (str): A correlation ID associated with requests related to this event. [optional]  # noqa: E501
            session (str): The session associated with tokens related to this event. This can be used to tie the actions undertaking by requests bearing tokens with the same session back to the authentication events which created the tokens. . [optional]  # noqa: E501
            issuer (str): The issuer the user logged in to.. [optional]  # noqa: E501
            client_id (str): The client id of the web application, client, etc. that the user is logging in with. Note that this is not the id of the `IssuerClient`, but rather the id presented to the authentication system to identify that client. This corresponds to `name` in the `IssuerClient`. . [optional]  # noqa: E501
            application_name (str): The name of the application within the system the user is logging in to.. [optional]  # noqa: E501
            login_org_id (str): The id of the organisation that the user is logging in to. Note that this is disctinct from the `org_id` field, which is tied to the issuer. This id is tied to the application. . [optional]  # noqa: E501
            login_org_name (str): The name of the organisation that the user is logging in to. This corresponds to `login_org_id`. . [optional]  # noqa: E501
            upstream_idp (str): The name of the identity provider proving the identity of the user. . [optional]  # noqa: E501
            stage (str): The stage of the login process. This identifies where in the pipeline the event was generated. Deprecated, this has been replaced with event_name. . [optional]  # noqa: E501
            result (str): The result of the event. The following results are supported:   \"Success\"   \"InProgress\"   \"Authenticate\"   \"DoMfa\"   \"Create\"   \"Failure\"   \"Denied\"   \"Timeout\"   \"CodeIssued\"   \"SessionExpired\" . [optional]  # noqa: E501
            user_agent (str): The user agent of the client used to perform the login. . [optional]  # noqa: E501
            request_id (str): The unique ID linking multiple parts of an authentication flow together. Use this to correlate multiple audit records to understand the end result of a single login attempt. . [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    def __python_set(val):
        return set(val)
 
    required_properties = __python_set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """AuthAudits - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            user_id (str): The system-local id of the user performing the action.. [optional]  # noqa: E501
            upstream_user_id (str): The id of the user in the upstream system, if available.. [optional]  # noqa: E501
            org_id (str): The id of the organisation of the issuer against which the user is authenticating. . [optional]  # noqa: E501
            org_name (str): The name of the organisation of the issuer against which the user is authenticating. . [optional]  # noqa: E501
            time (datetime): the time at which the record was generated.. [optional]  # noqa: E501
            event (str): The event which generated the record. The meaning of the event depends on the stage where it occured. Deprecated: this has been replaced with Result. . [optional]  # noqa: E501
            event_name (str): The event name which generated the record. The following events are supported:    \"Authentication Request\":  An OIDC 3-way handshake authentication request flow (RFC6749)    \"Identity Provider Authentication\": Authentication with an upstream identity provider.    \"Identity Provider Callback\": A callback received during the OIDC 3-way handshake that involves     communicating with the upstream identity provider.    \"Assertion Request\":  Authorization associated with a service account utilizing     an authentication document.    \"Policy Evaluation\": As part of authentication and authorization, policy is being evaluated.    \"Token\": Creation of an Access Token for access to Agilicus resources.    \"MFA Challenge\": A Multifactor method is required and this is communicating to the user     to invoke their MFA method. . [optional]  # noqa: E501
            source_ip (str): The IP address of the host initiating the action. [optional]  # noqa: E501
            token_id (str): The id of the token issued or reissued as part of the authentication.. [optional]  # noqa: E501
            trace_id (str): A correlation ID associated with requests related to this event. [optional]  # noqa: E501
            session (str): The session associated with tokens related to this event. This can be used to tie the actions undertaking by requests bearing tokens with the same session back to the authentication events which created the tokens. . [optional]  # noqa: E501
            issuer (str): The issuer the user logged in to.. [optional]  # noqa: E501
            client_id (str): The client id of the web application, client, etc. that the user is logging in with. Note that this is not the id of the `IssuerClient`, but rather the id presented to the authentication system to identify that client. This corresponds to `name` in the `IssuerClient`. . [optional]  # noqa: E501
            application_name (str): The name of the application within the system the user is logging in to.. [optional]  # noqa: E501
            login_org_id (str): The id of the organisation that the user is logging in to. Note that this is disctinct from the `org_id` field, which is tied to the issuer. This id is tied to the application. . [optional]  # noqa: E501
            login_org_name (str): The name of the organisation that the user is logging in to. This corresponds to `login_org_id`. . [optional]  # noqa: E501
            upstream_idp (str): The name of the identity provider proving the identity of the user. . [optional]  # noqa: E501
            stage (str): The stage of the login process. This identifies where in the pipeline the event was generated. Deprecated, this has been replaced with event_name. . [optional]  # noqa: E501
            result (str): The result of the event. The following results are supported:   \"Success\"   \"InProgress\"   \"Authenticate\"   \"DoMfa\"   \"Create\"   \"Failure\"   \"Denied\"   \"Timeout\"   \"CodeIssued\"   \"SessionExpired\" . [optional]  # noqa: E501
            user_agent (str): The user agent of the client used to perform the login. . [optional]  # noqa: E501
            request_id (str): The unique ID linking multiple parts of an authentication flow together. Use this to correlate multiple audit records to understand the end result of a single login attempt. . [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")




# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Copyright 2022 Agilicus Inc. All rights reserved

"""Utilities for OAuth.

Utilities for making it easier to work with OAuth 2.0
credentials.
"""

import os
import threading

from oauth2client import _helpers
from oauth2client import client
from .keyring import KeyringEncrypter


class Storage(client.Storage):
    """Store and retrieve a single credential to and from an encrypted file."""

    def __init__(self, filename):
        super(Storage, self).__init__(lock=threading.Lock())
        self._filename = filename
        self.encrypter = KeyringEncrypter()

    def locked_get(self):
        """Retrieve Credential from file.

        Returns:
            oauth2client.client.Credentials

        Raises:
            IOError if the file is a symbolic link.
        """
        credentials = None
        _helpers.validate_file(self._filename)
        try:
            f = open(self._filename, "rb")
            enc_content = f.read()
            f.close()
        except IOError:
            return credentials

        try:
            content = self.encrypter.decrypt(enc_content)
            credentials = client.Credentials.new_from_json(content)
            credentials.set_store(self)
        except ValueError:
            pass

        return credentials

    def _create_file_if_needed(self):
        """Create an empty file if necessary.

        This method will not initialize the file. Instead it implements a
        simple version of "touch" to ensure the file has been created.
        """
        if not os.path.exists(self._filename):
            old_umask = os.umask(0o177)
            try:
                open(self._filename, "a+b").close()
            finally:
                os.umask(old_umask)

    def locked_put(self, credentials):
        """Write Credentials to file.

        Args:
            credentials: Credentials, the credentials to store.

        Raises:
            IOError if the file is a symbolic link.
        """
        self._create_file_if_needed()
        _helpers.validate_file(self._filename)
        f = open(self._filename, "wb")
        enc_content = self.encrypter.encrypt(credentials.to_json())
        f.write(enc_content)
        f.close()

    def locked_delete(self):
        """Delete Credentials file.

        Args:
            credentials: Credentials, the credentials to store.
        """
        os.unlink(self._filename)



import sys
import agilicus_api
from prettytable import PrettyTable

import textwrap

from . import billing
from . import context
from . import connectors


def override_replace(metric, usage_override, usage_min, usage_max, usage_step):
    if usage_override is None:
        usage_override = []
    usage_min = None if usage_min is None else int(usage_min)
    usage_max = None if usage_max is None else int(usage_max)
    usage_step = None if usage_step is None else int(usage_step)
    for idx, x in enumerate(usage_override):
        if x["metric"] == metric:
            usage_override.remove(x)
    if usage_min is not None or usage_max or usage_step is not None:
        rec = {"metric": metric}
        if usage_min is not None:
            rec["min_quantity"] = usage_min
        if usage_max is not None:
            rec["max_quantity"] = usage_max
        if usage_step is not None:
            rec["step_size"] = usage_step
        usage_override.append(rec)
    return usage_override


def set_subscription_info(ctx, **kwargs):
    """Update various parameters associated with the subscription.
    e.g. the usage-metrics of min/max/step size."""
    result = billing.list_subscriptions(ctx, org_id=kwargs["org_id"])

    for bi in result.billing_subscriptions:
        billing_account_id = bi.spec.billing_account_id
        for org in bi.status.orgs:
            if org.id == kwargs["org_id"]:
                subscription_id = bi.metadata.id
                break

    if subscription_id is None:
        print(f"ERROR: could not find account info for {kwargs['org_id']}")
        sys.exit(1)

    subscription = billing.get_billing_subscription(
        ctx, billing_subscription_id=subscription_id
    )
    subscription_id = subscription.metadata.id
    usage_override = subscription.spec.usage_override

    usage_override = override_replace(
        "active_users",
        usage_override,
        kwargs["min_user"],
        kwargs["max_user"],
        kwargs["step_user"],
    )
    usage_override = override_replace(
        "active_connectors",
        usage_override,
        kwargs["min_connector"],
        kwargs["max_connector"],
        kwargs["step_connector"],
    )

    subscription.spec.usage_override = usage_override

    billing.update_subscription(
        ctx, billing_subscription_id=subscription_id, subscription=subscription
    )
    res = billing.create_usage_record(ctx, billing_account_id=billing_account_id)
    print(res)


def _show_billing(ctx, org, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    billing_account = apiclient.billing_api.get_billing_account(
        billing_account_id=org.billing_account_id
    )
    if kwargs["markdown"]:
        dl = "| "
    else:
        dl = " "
    if "billing_subscription_id" in org and org.billing_subscription_id is not None:
        subscription = apiclient.billing_api.get_subscription(
            billing_subscription_id=org.billing_subscription_id
        )

        cid = billing_account.spec.customer_id
        sid = subscription.spec.subscription_id
        surl = "https://dashboard.stripe.com"
        if kwargs["markdown"]:
            print(f"{dl}STRIPE ACCOUNT:   {dl}{cid:30} [link]({surl}/customers/{cid})")
            print(
                f"{dl}STRIPE SUBSCRIPT: {dl}{sid:30} [link]({surl}/subscriptions/{sid})"
            )
        else:
            print(f"{dl}STRIPE ACCOUNT:   {dl}{cid:30} {surl}/customers/{cid}")
            print(f"{dl}STRIPE SUBSCRIPT: {dl}{sid:30} {surl}/subscriptions/{sid}")


def _show_usage(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    metrics = apiclient.org_api.get_usage_metrics(org_id=kwargs["org_id"]).metrics
    if kwargs["markdown"]:
        dl = "| "
        print("")
    else:
        dl = " "
        print("USAGE")

    print(f"{dl}METRIC               {dl}PEAK    {dl}CURRENT")
    if kwargs["markdown"]:
        print(f"{dl}------               {dl}----    {dl}-------")

    for metric in metrics:
        tname = metric.type
        if metric.type == "fileshare":
            tname = "share"
        print(
            f"{dl}{tname:20} {dl}{metric.provisioned.peak:5}   {dl}{metric.provisioned.current:5}"  # noqa: E501
        )


def _get_last_auth(ctx, kwargs):
    dt_from = "-90d"
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    query_results = apiclient.audits_api.list_auth_records(
        dt_from=dt_from,
        org_id=kwargs["org_id"],
        event="Success",
        result="Success",
        event_name="Authentication Request",
        limit=100,
    )
    for rx in query_results.auth_audits:
        if rx.event == "Success" and rx.result == "Success":
            return rx.time.strftime("%Y-%m-%d %H:%M:%S")
    return None


def _show_auth(ctx, **kwargs):
    """Sign-in usage."""
    dt_from = "-30d"
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    query_results = apiclient.audits_api.list_auth_records(
        dt_from=dt_from,
        org_id=kwargs["org_id"],
        event_name="Authentication Request",
        limit=500,
    )
    odate = None
    pids = {}
    for rx in query_results.auth_audits:
        if odate is None or rx.time > odate:
            odate = rx.time
        if rx.event == "Success" and rx.result == "Success":
            pids[rx.client_id] = pids.get(rx.client_id, 0) + 1
    if kwargs["markdown"]:
        dl = "| "
        print("")
    else:
        dl = " "
    if len(pids):
        print("\n**APPROXIMATE AUTHENTICATION**\n")

        print(f"{dl}CLIENT-ID                             {dl}AUTHS")
        if kwargs["markdown"]:
            print("| ------------------------------------- | -----")
        for p in pids:
            print(f"{dl}{p:35}   {dl}{pids[p]:5}")
    else:
        print("NO AUTHENTICATION ACTIONS")
    if odate is None:
        print("\nNO AUTHENTICATION AUDIT IN TRAILING 30d")
    else:
        print(f"\nNEWEST AUTH RECORD: {odate.isoformat()}")


def remfl(s):
    s = s.split("\n", 1)[-1]
    if s.find("\n") == -1:
        return ""
    return s.rsplit("\n", 1)[0]


def _show_connectors(ctx, **kwargs):
    kw = {}
    kw["sort_by"] = ["metadata.created"]
    kw["org_id"] = kwargs["org_id"]
    kw["show_stats"] = True
    results = connectors.query(ctx, **kw)
    table = connectors.format_connectors_as_text(ctx, results, skip_sub_table=True, **kw)
    raw = table.get_string()

    if kwargs["markdown"]:
        print("\n**CONNECTORS**\n")
        print(remfl(raw).replace("+", "|"))
        print("")
    else:
        print("\nCONNECTORS")
        print(textwrap.indent(raw, "  "))


def _show_status(ctx, org, **kwargs):
    if org is None:
        token = context.get_token(ctx)
        apiclient = context.get_apiclient(ctx, token)
        org_id = kwargs["org_id"]
        org = apiclient.org_api.get_org(org_id=org_id)
    if kwargs["markdown"]:
        dl = "| "
        print("| USAGE | VALUES")
        print("| ----- | -----")
    else:
        dl = ""

    print(f"{dl}NAME:    {dl}{org.organisation}")
    print(f"{dl}ID:      {dl}{org.id}")
    print(f"{dl}CREATED: {dl}{org.created}")
    print(f"{dl}SHARD:   {dl}{org.shard}")
    print(f"{dl}ISSUER:  {dl}{org.issuer}")
    print(f"{dl}STATUS:  {dl}{org.admin_state}")
    print(f"{dl}CLUSTER: {dl}{org.cluster}")
    kwargs["org_id"] = org.id
    _show_billing(ctx, org, **kwargs)
    _show_usage(ctx, **kwargs)
    _show_auth(ctx, **kwargs)
    _show_connectors(ctx, **kwargs)


def status(ctx, **kwargs):
    """Show snapshot of this org, trial, billing, usage, status."""
    if "org_id" in kwargs and kwargs["org_id"] is not None:
        _show_status(ctx, org=None, **kwargs)
    if kwargs["email"]:
        token = context.get_token(ctx)
        apiclient = context.get_apiclient(ctx, token)
        n_orgs = 1
        page_at_id = ""
        while n_orgs > 0:
            params = {}
            params["page_at_id"] = page_at_id
            params["limit"] = 500
            try:
                orgs = apiclient.org_api.list_orgs(**params)
            except agilicus_api.exceptions.UnauthorizedException:
                orgs = apiclient.org_api.list_orgs(**params)

            n_orgs = 0
            try:
                orgs = apiclient.org_api.list_orgs(**params)
            except agilicus_api.exceptions.UnauthorizedException:
                orgs = apiclient.org_api.list_orgs(**params)
            for org in orgs.orgs:
                n_orgs += 1
                page_at_id = org.id
                if org.contact_email == kwargs["email"]:
                    _show_status(ctx, org=org, **kwargs)


def all_org_status(ctx, **kwargs):
    """Show the current status of all orgs with parent as parend-id."""
    params = {}
    params["org_id"] = kwargs["parent_org_id"]
    params["enabled"] = True
    params["limit"] = 60
    params["list_children"] = True

    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    table = PrettyTable()
    table.align = "l"
    table.field_name = ["Num", "ID", "Name", "Last-Auth"]
    table.align["Num"] = "r"

    num = 0
    n_orgs = 1
    page_at_id = ""
    while n_orgs > 0:
        params["page_at_id"] = page_at_id
        n_orgs = 0
        try:
            orgs = apiclient.org_api.list_orgs(**params)
        except agilicus_api.exceptions.UnauthorizedException:
            orgs = apiclient.org_api.list_orgs(**params)
        for org in orgs.orgs:
            n_orgs += 1
            page_at_id = org.id
            if org.parent_id == kwargs["parent_org_id"]:
                num = num + 1
                args = {}
                args["org_id"] = org.id
                odate = _get_last_auth(ctx, args)
                table.add_row([num, org.id, org.organisation, odate])

    print(table)



from unittest.mock import MagicMock

class ApplicationsApiMock:

    def __init__(self):
        self.mock_add_config = MagicMock()
        self.mock_add_role = MagicMock()
        self.mock_add_role_to_rule_entry = MagicMock()
        self.mock_add_rule = MagicMock()
        self.mock_create_application = MagicMock()
        self.mock_delete_application = MagicMock()
        self.mock_delete_config = MagicMock()
        self.mock_delete_role = MagicMock()
        self.mock_delete_role_to_rule_entry = MagicMock()
        self.mock_delete_rule = MagicMock()
        self.mock_get_all_usage_metrics = MagicMock()
        self.mock_get_application = MagicMock()
        self.mock_get_application_usage_metrics = MagicMock()
        self.mock_get_config = MagicMock()
        self.mock_get_environment = MagicMock()
        self.mock_get_role = MagicMock()
        self.mock_get_role_to_rule_entry = MagicMock()
        self.mock_get_rule = MagicMock()
        self.mock_list_application_summaries = MagicMock()
        self.mock_list_applications = MagicMock()
        self.mock_list_combined_rules = MagicMock()
        self.mock_list_configs = MagicMock()
        self.mock_list_environment_configs_all_apps = MagicMock()
        self.mock_list_role_to_rule_entries = MagicMock()
        self.mock_list_roles = MagicMock()
        self.mock_list_rules = MagicMock()
        self.mock_list_runtime_status = MagicMock()
        self.mock_replace_application = MagicMock()
        self.mock_replace_config = MagicMock()
        self.mock_replace_environment = MagicMock()
        self.mock_replace_role = MagicMock()
        self.mock_replace_role_to_rule_entry = MagicMock()
        self.mock_replace_rule = MagicMock()
        self.mock_replace_runtime_status = MagicMock()
        self.mock_update_patch_application = MagicMock()

    def add_config(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.add_config with MagicMock.
        """
        return self.mock_add_config(self, *args, **kwargs)

    def add_role(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.add_role with MagicMock.
        """
        return self.mock_add_role(self, *args, **kwargs)

    def add_role_to_rule_entry(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.add_role_to_rule_entry with MagicMock.
        """
        return self.mock_add_role_to_rule_entry(self, *args, **kwargs)

    def add_rule(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.add_rule with MagicMock.
        """
        return self.mock_add_rule(self, *args, **kwargs)

    def create_application(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.create_application with MagicMock.
        """
        return self.mock_create_application(self, *args, **kwargs)

    def delete_application(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.delete_application with MagicMock.
        """
        return self.mock_delete_application(self, *args, **kwargs)

    def delete_config(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.delete_config with MagicMock.
        """
        return self.mock_delete_config(self, *args, **kwargs)

    def delete_role(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.delete_role with MagicMock.
        """
        return self.mock_delete_role(self, *args, **kwargs)

    def delete_role_to_rule_entry(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.delete_role_to_rule_entry with MagicMock.
        """
        return self.mock_delete_role_to_rule_entry(self, *args, **kwargs)

    def delete_rule(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.delete_rule with MagicMock.
        """
        return self.mock_delete_rule(self, *args, **kwargs)

    def get_all_usage_metrics(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_all_usage_metrics with MagicMock.
        """
        return self.mock_get_all_usage_metrics(self, *args, **kwargs)

    def get_application(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_application with MagicMock.
        """
        return self.mock_get_application(self, *args, **kwargs)

    def get_application_usage_metrics(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_application_usage_metrics with MagicMock.
        """
        return self.mock_get_application_usage_metrics(self, *args, **kwargs)

    def get_config(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_config with MagicMock.
        """
        return self.mock_get_config(self, *args, **kwargs)

    def get_environment(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_environment with MagicMock.
        """
        return self.mock_get_environment(self, *args, **kwargs)

    def get_role(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_role with MagicMock.
        """
        return self.mock_get_role(self, *args, **kwargs)

    def get_role_to_rule_entry(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_role_to_rule_entry with MagicMock.
        """
        return self.mock_get_role_to_rule_entry(self, *args, **kwargs)

    def get_rule(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.get_rule with MagicMock.
        """
        return self.mock_get_rule(self, *args, **kwargs)

    def list_application_summaries(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_application_summaries with MagicMock.
        """
        return self.mock_list_application_summaries(self, *args, **kwargs)

    def list_applications(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_applications with MagicMock.
        """
        return self.mock_list_applications(self, *args, **kwargs)

    def list_combined_rules(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_combined_rules with MagicMock.
        """
        return self.mock_list_combined_rules(self, *args, **kwargs)

    def list_configs(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_configs with MagicMock.
        """
        return self.mock_list_configs(self, *args, **kwargs)

    def list_environment_configs_all_apps(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_environment_configs_all_apps with MagicMock.
        """
        return self.mock_list_environment_configs_all_apps(self, *args, **kwargs)

    def list_role_to_rule_entries(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_role_to_rule_entries with MagicMock.
        """
        return self.mock_list_role_to_rule_entries(self, *args, **kwargs)

    def list_roles(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_roles with MagicMock.
        """
        return self.mock_list_roles(self, *args, **kwargs)

    def list_rules(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_rules with MagicMock.
        """
        return self.mock_list_rules(self, *args, **kwargs)

    def list_runtime_status(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.list_runtime_status with MagicMock.
        """
        return self.mock_list_runtime_status(self, *args, **kwargs)

    def replace_application(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_application with MagicMock.
        """
        return self.mock_replace_application(self, *args, **kwargs)

    def replace_config(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_config with MagicMock.
        """
        return self.mock_replace_config(self, *args, **kwargs)

    def replace_environment(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_environment with MagicMock.
        """
        return self.mock_replace_environment(self, *args, **kwargs)

    def replace_role(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_role with MagicMock.
        """
        return self.mock_replace_role(self, *args, **kwargs)

    def replace_role_to_rule_entry(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_role_to_rule_entry with MagicMock.
        """
        return self.mock_replace_role_to_rule_entry(self, *args, **kwargs)

    def replace_rule(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_rule with MagicMock.
        """
        return self.mock_replace_rule(self, *args, **kwargs)

    def replace_runtime_status(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.replace_runtime_status with MagicMock.
        """
        return self.mock_replace_runtime_status(self, *args, **kwargs)

    def update_patch_application(self, *args, **kwargs):
        """
        This method mocks the original api ApplicationsApi.update_patch_application with MagicMock.
        """
        return self.mock_update_patch_application(self, *args, **kwargs)




import click
from ..output.table import output_entry
from . import policy_config


@click.command(name="get-authz-bundle")
@click.pass_context
@click.option("--org-id", default=None)
@click.option("--policy-config-etag", default=None)
def cli_command_get_authz_bundle(ctx, **kwargs):
    result = policy_config.get_authz_bundle(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="delete-authz-bundle")
@click.pass_context
@click.option("--org-id", default=None)
def cli_command_delete_authz_bundle(ctx, **kwargs):
    policy_config.delete_authz_bundle(ctx, **kwargs)


all_funcs = [func for func in dir() if "cli_command_" in func]


def add_commands(cli):
    glob = globals()
    for func in all_funcs:
        cli.add_command(glob[func])



from ..input_helpers import (
    get_org_from_input_or_ctx,
    strip_none,
)

from .. import context

from agilicus import (
    create_or_update,
    LabelName,
    PolicyTemplateInstance,
    PolicyTemplateInstanceSpec,
    MFAPolicyTemplate,
)

from ..output.table import (
    format_table,
    spec_column,
    metadata_column,
    subtable,
    column,
)


class InstanceAddInfo:
    def __init__(self, apiclient):
        super().__init__()


def set_multifactor_policy(ctx, name, duration, label=None, description=None, **kwargs):
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)

    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    mfa = MFAPolicyTemplate(
        seconds_since_last_challenge=duration,
        labels=[LabelName(la) for la in (label or [])],
        template_type="mfa",
    )

    spec = PolicyTemplateInstanceSpec(
        org_id=org_id,
        name=name,
        template=mfa,
    )

    if description is not None:
        spec.description = description

    tmpl = PolicyTemplateInstance(spec=spec)
    templates_api = apiclient.policy_templates_api
    resp, _ = create_or_update(
        tmpl,
        lambda obj: templates_api.create_policy_template_instance(obj),
        lambda guid, obj: templates_api.replace_policy_template_instance(
            guid, policy_template_instance=obj
        ),
        to_dict=False,
    )
    return resp


def ruleset_labelled(ruleset, label):
    for ruleset_label in ruleset.spec.labels or []:
        if str(ruleset_label) == label:
            return True
    return False


def list_multifactor_policies(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    result = apiclient.policy_templates_api.list_policy_template_instances(
        org_id=org_id, template_type="mfa"
    )
    return result.policy_template_instances


def format_multifactor_policies(ctx, templates):
    mfa_columns = [
        column("seconds_since_last_challenge"),
        column("labels"),
    ]
    mfa_table = subtable(ctx, "spec.template", mfa_columns)
    columns = [
        spec_column("org_id"),
        spec_column("name"),
        spec_column("template.template_type", "type"),
        mfa_table,
    ]

    return format_table(ctx, templates, columns)


def list_policy_templates(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    result = apiclient.policy_templates_api.list_policy_template_instances(**kwargs)
    return result.policy_template_instances


def delete_policy_template(ctx, instance_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    apiclient.policy_templates_api.delete_policy_template_instance(instance_id, **kwargs)


def get_policy_template(ctx, instance_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    return apiclient.policy_templates_api.get_policy_template_instance(
        instance_id, **kwargs
    )


def format_policy_templates(ctx, templates):
    columns = [
        spec_column("org_id"),
        metadata_column("id"),
        spec_column("name"),
        spec_column("template.template_type", "type"),
        spec_column("description"),
        spec_column("template"),
    ]

    return format_table(ctx, templates, columns)



# flake8: noqa
from typing import Callable
import json
import os
import sys
import dataclasses
from . import context
from . import scopes
from . import credentials
from . import tokens
import jwt

from .version import __version__  # noqa

sys.path.append(os.path.dirname(__file__))  # noqa

from .agilicus_api import *  # noqa
from .agilicus_api import exceptions  # noqa
from . import patches  # noqa
from .pagination.pagination import get_many_entries

ApiClient = patches.patched_api_client()


def find_guid(obj: dict):
    guid = obj.get("id", None)
    if guid:
        return guid
    md = obj.get("metadata", {})
    guid = md.get("id", None)
    if guid:
        return guid
    raise Exception(f"GUID cannot be found in obj {obj}")


def create_or_update(
    obj,
    create_method: Callable,
    update_method: Callable = None,
    to_dict=True,
    guid_finder=find_guid,
) -> [dict, str]:
    """A helper method that handles duplicate (409)
    creation of objects. On 409, if the update_method is provided,
    it will apply a PUT on the resource to update it with
    the new requested data. The guid is searched for in the object
    returned from the 409, and this is then provided
    to the update method as an argument, along with the original
    object that should be applied.
        param: obj: the object to be created or updated
        param: create_method(obj, ...)
        param: update_method(guid, obj, ...)
        returns: tuple, the object, with the status code.

        Note the status code could be:
           409: a duplicate, no update was performed
           201: a create was succesfully made
           200: a duplicate occured, and the object was updated accordingly
    """
    result = None
    try:
        result = create_method(obj)
        if to_dict:
            return result.to_dict(), 201
        else:
            return result, 201
    except ApiException as exc:
        if exc.status == 409:
            body = exc.body
            if not body:
                raise
            result = json.loads(body)
            if update_method:
                guid = guid_finder(result)
                if to_dict:
                    return update_method(guid, obj).to_dict(), 200
                else:
                    return update_method(guid, obj), 200
            else:
                return result, 409
        else:
            raise
    return result


def GetClient(
    issuer=context.ISSUER_DEFAULT,
    cacert=None,
    client_id="agilicus-builtin-cli",
    authentication_document=None,
    agilicus_scopes=scopes.DEFAULT_SCOPES,
    auth_local_webserver=True,
    api_url=None,
    expiry=None,
):

    config = Configuration(host=api_url, ssl_ca_cert=cacert)
    if authentication_document:
        creds = {}
        with open(authentication_document) as fd:
            ad = json.load(fd)
        token = tokens.create_service_token(
            auth_doc=ad,
            scope=agilicus_scopes,
            client_id=client_id,
            expiry=expiry,
            verify=cacert,
        )
        config.access_token = token.get("access_token")
    else:
        creds = credentials.get_credentials(
            issuer=issuer,
            cacert=cacert,
            client_id=client_id,
            agilicus_scopes=agilicus_scopes,
            auth_local_webserver=auth_local_webserver,
        )
        config.access_token = creds.access_token

    _default_org_id = None
    access_token = jwt.decode(
        config.access_token,
        algorithms=["ES256"],
        options={"verify_signature": False},
        leeway=60,
    )
    if "org" in access_token:
        _default_org_id = access_token["org"]

    @dataclasses.dataclass
    class api:
        default_org_id = _default_org_id
        users = UsersApi(ApiClient(config))
        billing = BillingApi(ApiClient(config))
        organisations = OrganisationsApi(ApiClient(config))
        policies = PolicyApi(ApiClient(config))
        certificates = CertificatesApi(ApiClient(config))
        applications = ApplicationsApi(ApiClient(config))
        groups = GroupsApi(ApiClient(config))
        connectors = ConnectorsApi(ApiClient(config))
        resouces = ResourcesApi(ApiClient(config))
        catalogues = CataloguesApi(ApiClient(config))
        permissions = PermissionsApi(ApiClient(config))
        audits = AuditsApi(ApiClient(config))
        files = FilesApi(ApiClient(config))
        tokens = TokensApi(ApiClient(config))
        diagnostics = DiagnosticsApi(ApiClient(config))
        metrics = MetricsApi(ApiClient(config))
        challenges = ChallengesApi(ApiClient(config))
        application_services = ApplicationServicesApi(ApiClient(config))
        issuers = IssuersApi(ApiClient(config))
        messages = MessagesApi(ApiClient(config))
        lookups = LookupsApi(ApiClient(config))
        trusted_certs = TrustedCertsApi(ApiClient(config))
        rules = RulesApi(ApiClient(config))
        policy_config = PolicyConfigApi(ApiClient(config))

    return api



import agilicus
import argparse
import sys

scopes = agilicus.scopes.DEFAULT_SCOPES

parser = argparse.ArgumentParser(description="update-user")
parser.add_argument("--auth-doc", type=str)
parser.add_argument("--issuer", type=str)
parser.add_argument("--email", type=str)
parser.add_argument("--disable-user", type=bool, default=None)
args = parser.parse_args()

if not args.auth_doc and not args.issuer:
    print("error: specify either an --auth-doc or --issuer")
    sys.exit(1)

if not args.email:
    print("error: specify an email to search for a user")
    sys.exit(1)

api = agilicus.GetClient(
    agilicus_scopes=scopes, issuer=args.issuer, authentication_document=args.auth_doc
)

users = api.users.list_users(org_id=api.default_org_id, email=args.email)
if len(users.users) != 1:
    print(f"error: failed to find user with email: {args.email}")
    sys.exit(1)

user = users.users[0]

if args.disable_user is not None:
    user.enabled = args.disable_user

result = api.users.replace_user(
    user.id, user=user, _check_input_type=False, _host_index=0
)
print(result)



from ..input_helpers import get_org_from_input_or_ctx
from ..context import get_apiclient_from_ctx
from ..input_helpers import strip_none


def get_api(ctx):
    return get_apiclient_from_ctx(ctx).policy_config_api


def get_authz_bundle(ctx, org_id, **kwargs):
    api = get_api(ctx)
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)

    return api.get_authz_bundle(org_id, **strip_none(kwargs))


def delete_authz_bundle(ctx, org_id, **kwargs):
    api = get_api(ctx)
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)

    return api.delete_authz_bundle(org_id)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.tokens_api import TokensApi  # noqa: E501


class TestTokensApi(unittest.TestCase):
    """TokensApi unit test stubs"""

    def setUp(self):
        self.api = TokensApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_api_key(self):
        """Test case for create_api_key

        Create an API Key  # noqa: E501
        """
        pass

    def test_create_api_key_introspection(self):
        """Test case for create_api_key_introspection

        Introspect an API Key  # noqa: E501
        """
        pass

    def test_create_authentication_document(self):
        """Test case for create_authentication_document

        Create a authentication document  # noqa: E501
        """
        pass

    def test_create_bulk_delete_session_task(self):
        """Test case for create_bulk_delete_session_task

        Delete a set of sessions  # noqa: E501
        """
        pass

    def test_create_bulk_revoke_session_task(self):
        """Test case for create_bulk_revoke_session_task

        Revoke a set of sessions  # noqa: E501
        """
        pass

    def test_create_bulk_revoke_token_task(self):
        """Test case for create_bulk_revoke_token_task

        Revoke a set of tokens  # noqa: E501
        """
        pass

    def test_create_introspect_token(self):
        """Test case for create_introspect_token

        Introspect a token  # noqa: E501
        """
        pass

    def test_create_introspect_token_all_sub_orgs(self):
        """Test case for create_introspect_token_all_sub_orgs

        Introspect a token in all sub orgs  # noqa: E501
        """
        pass

    def test_create_reissued_token(self):
        """Test case for create_reissued_token

        Issue a new token from another  # noqa: E501
        """
        pass

    def test_create_revoke_token_task(self):
        """Test case for create_revoke_token_task

        Revoke a token  # noqa: E501
        """
        pass

    def test_create_session(self):
        """Test case for create_session

        Create a session  # noqa: E501
        """
        pass

    def test_create_session_and_token(self):
        """Test case for create_session_and_token

        Create a session and a token associated with the session  # noqa: E501
        """
        pass

    def test_create_session_challenge(self):
        """Test case for create_session_challenge

        Create a user challenge request for a session  # noqa: E501
        """
        pass

    def test_create_token(self):
        """Test case for create_token

        Create a token  # noqa: E501
        """
        pass

    def test_create_token_validation(self):
        """Test case for create_token_validation

        Validate a token request  # noqa: E501
        """
        pass

    def test_create_user_data_token(self):
        """Test case for create_user_data_token

        Create a User Data Token  # noqa: E501
        """
        pass

    def test_delete_api_key(self):
        """Test case for delete_api_key

        Delete an API Key  # noqa: E501
        """
        pass

    def test_delete_authentication_document(self):
        """Test case for delete_authentication_document

        Delete a authentication document  # noqa: E501
        """
        pass

    def test_delete_session(self):
        """Test case for delete_session

        Delete a session  # noqa: E501
        """
        pass

    def test_get_api_key(self):
        """Test case for get_api_key

        Get an API Key  # noqa: E501
        """
        pass

    def test_get_authentication_document(self):
        """Test case for get_authentication_document

        Get a authentication document  # noqa: E501
        """
        pass

    def test_get_jwks(self):
        """Test case for get_jwks

        Return JSON Web Key Set (JWKS)  # noqa: E501
        """
        pass

    def test_get_session(self):
        """Test case for get_session

        Get a session  # noqa: E501
        """
        pass

    def test_get_token(self):
        """Test case for get_token

        introspect token  # noqa: E501
        """
        pass

    def test_get_user_data_jwks(self):
        """Test case for get_user_data_jwks

        Return JSON Web Key Set (JWKS)  # noqa: E501
        """
        pass

    def test_list_api_keys(self):
        """Test case for list_api_keys

        List API Keys  # noqa: E501
        """
        pass

    def test_list_authentication_documents(self):
        """Test case for list_authentication_documents

        List authentication documents  # noqa: E501
        """
        pass

    def test_list_sessions(self):
        """Test case for list_sessions

        List Sessions  # noqa: E501
        """
        pass

    def test_list_tokens(self):
        """Test case for list_tokens

        Query tokens  # noqa: E501
        """
        pass

    def test_replace_api_key(self):
        """Test case for replace_api_key

        Update an API key  # noqa: E501
        """
        pass

    def test_replace_session(self):
        """Test case for replace_session

        Update a session  # noqa: E501
        """
        pass

    def test_update_session_challenge(self):
        """Test case for update_session_challenge

        Update a session with its challenge answer  # noqa: E501
        """
        pass

    def test_validate_identity_assertion(self):
        """Test case for validate_identity_assertion

        Validate an identity assertion  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.api_client import ApiClient, Endpoint as _Endpoint
from agilicus_api.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.list_trusted_certificate import ListTrustedCertificate
from agilicus_api.model.list_trusted_certificate_bundle import ListTrustedCertificateBundle
from agilicus_api.model.list_trusted_certificate_label import ListTrustedCertificateLabel
from agilicus_api.model.list_trusted_certificate_org import ListTrustedCertificateOrg
from agilicus_api.model.trusted_certificate import TrustedCertificate
from agilicus_api.model.trusted_certificate_bundle import TrustedCertificateBundle
from agilicus_api.model.trusted_certificate_bundle_name import TrustedCertificateBundleName
from agilicus_api.model.trusted_certificate_label import TrustedCertificateLabel
from agilicus_api.model.trusted_certificate_label_name import TrustedCertificateLabelName


class TrustedCertsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __create_bundle(
            self,
            trusted_certificate_bundle,
            **kwargs
        ):
            """Creates a TrustedCertificateBundle  # noqa: E501

            Creates a TrustedCertificateBundle   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_bundle(trusted_certificate_bundle, async_req=True)
            >>> result = thread.get()

            Args:
                trusted_certificate_bundle (TrustedCertificateBundle):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificateBundle
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['trusted_certificate_bundle'] = \
                trusted_certificate_bundle
            return self.call_with_http_info(**kwargs)

        if self.create_bundle is None:
            self.create_bundle = _Endpoint(
                settings={
                    'response_type': (TrustedCertificateBundle,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_bundles',
                    'operation_id': 'create_bundle',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'trusted_certificate_bundle',
                    ],
                    'required': [
                        'trusted_certificate_bundle',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'trusted_certificate_bundle':
                            (TrustedCertificateBundle,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'trusted_certificate_bundle': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_bundle
            )

        def __create_label(
            self,
            trusted_certificate_label,
            **kwargs
        ):
            """Creates a TrustedCertificateLabel  # noqa: E501

            Creates a TrustedCertificateLabel   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_label(trusted_certificate_label, async_req=True)
            >>> result = thread.get()

            Args:
                trusted_certificate_label (TrustedCertificateLabel):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificateLabel
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['trusted_certificate_label'] = \
                trusted_certificate_label
            return self.call_with_http_info(**kwargs)

        if self.create_label is None:
            self.create_label = _Endpoint(
                settings={
                    'response_type': (TrustedCertificateLabel,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_labels',
                    'operation_id': 'create_label',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'trusted_certificate_label',
                    ],
                    'required': [
                        'trusted_certificate_label',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'trusted_certificate_label':
                            (TrustedCertificateLabel,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'trusted_certificate_label': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_label
            )

        def __create_trusted_cert(
            self,
            trusted_certificate,
            **kwargs
        ):
            """Creates a TrustedCertificate  # noqa: E501

            Creates a TrustedCertificate   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_trusted_cert(trusted_certificate, async_req=True)
            >>> result = thread.get()

            Args:
                trusted_certificate (TrustedCertificate):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificate
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['trusted_certificate'] = \
                trusted_certificate
            return self.call_with_http_info(**kwargs)

        if self.create_trusted_cert is None:
            self.create_trusted_cert = _Endpoint(
                settings={
                    'response_type': (TrustedCertificate,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs',
                    'operation_id': 'create_trusted_cert',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'trusted_certificate',
                    ],
                    'required': [
                        'trusted_certificate',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'trusted_certificate':
                            (TrustedCertificate,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'trusted_certificate': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_trusted_cert
            )

        def __delete_bundle(
            self,
            bundle,
            **kwargs
        ):
            """Delete a TrustedCertificateBundle  # noqa: E501

            Delete a TrustedCertificateBundle  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_bundle(bundle, async_req=True)
            >>> result = thread.get()

            Args:
                bundle (TrustedCertificateBundleName): A TrustedCertificateBundleName

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['bundle'] = \
                bundle
            return self.call_with_http_info(**kwargs)

        if self.delete_bundle is None:
            self.delete_bundle = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_bundles/{bundle}',
                    'operation_id': 'delete_bundle',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'bundle',
                        'org_id',
                    ],
                    'required': [
                        'bundle',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'bundle':
                            (TrustedCertificateBundleName,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'bundle': 'bundle',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'bundle': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_bundle
            )

        def __delete_label(
            self,
            label,
            **kwargs
        ):
            """Delete a TrustedCertificateLabel  # noqa: E501

            Delete a TrustedCertificateLabel  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_label(label, async_req=True)
            >>> result = thread.get()

            Args:
                label (TrustedCertificateLabelName): A TrustedCertificateLabelName

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label'] = \
                label
            return self.call_with_http_info(**kwargs)

        if self.delete_label is None:
            self.delete_label = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_labels/{label}',
                    'operation_id': 'delete_label',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label',
                        'org_id',
                    ],
                    'required': [
                        'label',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label':
                            (TrustedCertificateLabelName,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'label': 'label',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'label': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_label
            )

        def __delete_trusted_cert(
            self,
            certificate_id,
            **kwargs
        ):
            """Delete a TrustedCertificate  # noqa: E501

            Delete a TrustedCertificate  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_trusted_cert(certificate_id, async_req=True)
            >>> result = thread.get()

            Args:
                certificate_id (str): A certificate id

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['certificate_id'] = \
                certificate_id
            return self.call_with_http_info(**kwargs)

        if self.delete_trusted_cert is None:
            self.delete_trusted_cert = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs/{certificate_id}',
                    'operation_id': 'delete_trusted_cert',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'certificate_id',
                        'org_id',
                    ],
                    'required': [
                        'certificate_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'certificate_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'certificate_id': 'certificate_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'certificate_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_trusted_cert
            )

        def __get_bundle(
            self,
            bundle,
            **kwargs
        ):
            """Get a TrustedCertificateBundle  # noqa: E501

            Get a TrustedCertificateBundle  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_bundle(bundle, async_req=True)
            >>> result = thread.get()

            Args:
                bundle (TrustedCertificateBundleName): A TrustedCertificateBundleName

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                trusted_cert_bundle_etag (str): The entity tag (etag) for a requested bundle. If the returned etag matches the requested etag, then no data is returned, along with status code 304. . [optional]
                get_certs (bool): When querying a bundle, return all certs associated with bundle . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificateBundle
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['bundle'] = \
                bundle
            return self.call_with_http_info(**kwargs)

        if self.get_bundle is None:
            self.get_bundle = _Endpoint(
                settings={
                    'response_type': (TrustedCertificateBundle,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_bundles/{bundle}',
                    'operation_id': 'get_bundle',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'bundle',
                        'org_id',
                        'trusted_cert_bundle_etag',
                        'get_certs',
                    ],
                    'required': [
                        'bundle',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'bundle':
                            (TrustedCertificateBundleName,),
                        'org_id':
                            (str,),
                        'trusted_cert_bundle_etag':
                            (str,),
                        'get_certs':
                            (bool,),
                    },
                    'attribute_map': {
                        'bundle': 'bundle',
                        'org_id': 'org_id',
                        'trusted_cert_bundle_etag': 'trusted_cert_bundle_etag',
                        'get_certs': 'get_certs',
                    },
                    'location_map': {
                        'bundle': 'path',
                        'org_id': 'query',
                        'trusted_cert_bundle_etag': 'query',
                        'get_certs': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_bundle
            )

        def __get_label(
            self,
            label,
            **kwargs
        ):
            """Get a TrustedCertificateLabel  # noqa: E501

            Get a TrustedCertificateLabel  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_label(label, async_req=True)
            >>> result = thread.get()

            Args:
                label (TrustedCertificateLabelName): A TrustedCertificateLabelName

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificateLabel
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label'] = \
                label
            return self.call_with_http_info(**kwargs)

        if self.get_label is None:
            self.get_label = _Endpoint(
                settings={
                    'response_type': (TrustedCertificateLabel,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_labels/{label}',
                    'operation_id': 'get_label',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label',
                        'org_id',
                    ],
                    'required': [
                        'label',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label':
                            (TrustedCertificateLabelName,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'label': 'label',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'label': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_label
            )

        def __get_trusted_cert(
            self,
            certificate_id,
            **kwargs
        ):
            """Get a TrustedCertificate  # noqa: E501

            Get a TrustedCertificate  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_trusted_cert(certificate_id, async_req=True)
            >>> result = thread.get()

            Args:
                certificate_id (str): A certificate id

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificate
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['certificate_id'] = \
                certificate_id
            return self.call_with_http_info(**kwargs)

        if self.get_trusted_cert is None:
            self.get_trusted_cert = _Endpoint(
                settings={
                    'response_type': (TrustedCertificate,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs/{certificate_id}',
                    'operation_id': 'get_trusted_cert',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'certificate_id',
                        'org_id',
                    ],
                    'required': [
                        'certificate_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'certificate_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'certificate_id': 'certificate_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'certificate_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_trusted_cert
            )

        def __list_bundles(
            self,
            **kwargs
        ):
            """list TrustedCertificateBundle  # noqa: E501

            List TrustedCertificateBundle   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_bundles(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListTrustedCertificateBundle
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_bundles is None:
            self.list_bundles = _Endpoint(
                settings={
                    'response_type': (ListTrustedCertificateBundle,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_bundles',
                    'operation_id': 'list_bundles',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'limit': 'query',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_bundles
            )

        def __list_cert_orgs(
            self,
            **kwargs
        ):
            """list orgs that have available certificates  # noqa: E501

            List orgs that have available trusted certificates   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_cert_orgs(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListTrustedCertificateOrg
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_cert_orgs is None:
            self.list_cert_orgs = _Endpoint(
                settings={
                    'response_type': (ListTrustedCertificateOrg,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs/orgs',
                    'operation_id': 'list_cert_orgs',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_cert_orgs
            )

        def __list_labels(
            self,
            **kwargs
        ):
            """list TrustedCertificateLabel  # noqa: E501

            List TrustedCertificateLabel   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_labels(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                org_id (str): Organisation Unique identifier. [optional]
                label (str): Filters based on whether or not the items in the collection have the given label. . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListTrustedCertificateLabel
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_labels is None:
            self.list_labels = _Endpoint(
                settings={
                    'response_type': (ListTrustedCertificateLabel,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_labels',
                    'operation_id': 'list_labels',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'org_id',
                        'label',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'org_id':
                            (str,),
                        'label':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'org_id': 'org_id',
                        'label': 'label',
                    },
                    'location_map': {
                        'limit': 'query',
                        'org_id': 'query',
                        'label': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_labels
            )

        def __list_trusted_certs(
            self,
            **kwargs
        ):
            """list certificates  # noqa: E501

            List certificates   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_trusted_certs(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                org_id (str): Organisation Unique identifier. [optional]
                trusted_cert_label (TrustedCertificateLabelName): Query TrustedCertificates with a matching label name. [optional]
                page_at_id (str): Pagination based query with the id as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_id` field from the list response. . [optional]
                subject (str): Query a certificate based on subject name. This is an exact match query. . [optional]
                subject_search (str): Query a certificate based on subject name. Query for any certificates which contain this value as a case insensitive substring . [optional]
                subject_sha1 (str): Query a certificate based on the subject name sha128 . [optional]
                subject_sha256 (str): Query a certificate based on the subject name sha256 . [optional]
                cert_issuer (str): Query a certificate based on the issuer. . [optional]
                cert_issuer_search (str): Query a certificate based on the issuer Query for any certificates which contain this value as a case insensitive substring . [optional]
                serial_number (str): Query a certificate based on the serial number . [optional]
                skid (str): Query a certificate based on the subject key identifier . [optional]
                public_key_sha1 (str): Query a certificate based on the public key sha128 . [optional]
                public_key_sha256 (str): Query a certificate based on the public key sha256 . [optional]
                cert_root (bool): Query a certificates that are designated as a root . [optional]
                key_usage_extension_search (str): Query a certificates that have a matching key_usage_extension string. Query for any certificates which contain this value as a case insensitive substring . [optional]
                key_usage_crl_sign (bool): Query certificates that have key usage crl_sign . [optional]
                key_usage_key_cert_sign (bool): Query certificates that have key usage key_cert_sign . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListTrustedCertificate
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_trusted_certs is None:
            self.list_trusted_certs = _Endpoint(
                settings={
                    'response_type': (ListTrustedCertificate,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs',
                    'operation_id': 'list_trusted_certs',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'org_id',
                        'trusted_cert_label',
                        'page_at_id',
                        'subject',
                        'subject_search',
                        'subject_sha1',
                        'subject_sha256',
                        'cert_issuer',
                        'cert_issuer_search',
                        'serial_number',
                        'skid',
                        'public_key_sha1',
                        'public_key_sha256',
                        'cert_root',
                        'key_usage_extension_search',
                        'key_usage_crl_sign',
                        'key_usage_key_cert_sign',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'org_id':
                            (str,),
                        'trusted_cert_label':
                            (TrustedCertificateLabelName,),
                        'page_at_id':
                            (str,),
                        'subject':
                            (str,),
                        'subject_search':
                            (str,),
                        'subject_sha1':
                            (str,),
                        'subject_sha256':
                            (str,),
                        'cert_issuer':
                            (str,),
                        'cert_issuer_search':
                            (str,),
                        'serial_number':
                            (str,),
                        'skid':
                            (str,),
                        'public_key_sha1':
                            (str,),
                        'public_key_sha256':
                            (str,),
                        'cert_root':
                            (bool,),
                        'key_usage_extension_search':
                            (str,),
                        'key_usage_crl_sign':
                            (bool,),
                        'key_usage_key_cert_sign':
                            (bool,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'org_id': 'org_id',
                        'trusted_cert_label': 'trusted_cert_label',
                        'page_at_id': 'page_at_id',
                        'subject': 'subject',
                        'subject_search': 'subject_search',
                        'subject_sha1': 'subject_sha1',
                        'subject_sha256': 'subject_sha256',
                        'cert_issuer': 'cert_issuer',
                        'cert_issuer_search': 'cert_issuer_search',
                        'serial_number': 'serial_number',
                        'skid': 'skid',
                        'public_key_sha1': 'public_key_sha1',
                        'public_key_sha256': 'public_key_sha256',
                        'cert_root': 'cert_root',
                        'key_usage_extension_search': 'key_usage_extension_search',
                        'key_usage_crl_sign': 'key_usage_crl_sign',
                        'key_usage_key_cert_sign': 'key_usage_key_cert_sign',
                    },
                    'location_map': {
                        'limit': 'query',
                        'org_id': 'query',
                        'trusted_cert_label': 'query',
                        'page_at_id': 'query',
                        'subject': 'query',
                        'subject_search': 'query',
                        'subject_sha1': 'query',
                        'subject_sha256': 'query',
                        'cert_issuer': 'query',
                        'cert_issuer_search': 'query',
                        'serial_number': 'query',
                        'skid': 'query',
                        'public_key_sha1': 'query',
                        'public_key_sha256': 'query',
                        'cert_root': 'query',
                        'key_usage_extension_search': 'query',
                        'key_usage_crl_sign': 'query',
                        'key_usage_key_cert_sign': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_trusted_certs
            )

        def __replace_bundle(
            self,
            bundle,
            trusted_certificate_bundle,
            **kwargs
        ):
            """Update a TrustedCertificateBundle  # noqa: E501

            Update a TrustedCertificateBundle   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_bundle(bundle, trusted_certificate_bundle, async_req=True)
            >>> result = thread.get()

            Args:
                bundle (TrustedCertificateBundleName): A TrustedCertificateBundleName
                trusted_certificate_bundle (TrustedCertificateBundle):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificateBundle
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['bundle'] = \
                bundle
            kwargs['trusted_certificate_bundle'] = \
                trusted_certificate_bundle
            return self.call_with_http_info(**kwargs)

        if self.replace_bundle is None:
            self.replace_bundle = _Endpoint(
                settings={
                    'response_type': (TrustedCertificateBundle,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_cert_bundles/{bundle}',
                    'operation_id': 'replace_bundle',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'bundle',
                        'trusted_certificate_bundle',
                    ],
                    'required': [
                        'bundle',
                        'trusted_certificate_bundle',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'bundle':
                            (TrustedCertificateBundleName,),
                        'trusted_certificate_bundle':
                            (TrustedCertificateBundle,),
                    },
                    'attribute_map': {
                        'bundle': 'bundle',
                    },
                    'location_map': {
                        'bundle': 'path',
                        'trusted_certificate_bundle': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_bundle
            )

        def __replace_trusted_cert(
            self,
            certificate_id,
            trusted_certificate,
            **kwargs
        ):
            """Update a TrustedCertificate  # noqa: E501

            Update a TrustedCertificate   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_trusted_cert(certificate_id, trusted_certificate, async_req=True)
            >>> result = thread.get()

            Args:
                certificate_id (str): A certificate id
                trusted_certificate (TrustedCertificate):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                TrustedCertificate
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['certificate_id'] = \
                certificate_id
            kwargs['trusted_certificate'] = \
                trusted_certificate
            return self.call_with_http_info(**kwargs)

        if self.replace_trusted_cert is None:
            self.replace_trusted_cert = _Endpoint(
                settings={
                    'response_type': (TrustedCertificate,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/trusted_certs/{certificate_id}',
                    'operation_id': 'replace_trusted_cert',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'certificate_id',
                        'trusted_certificate',
                    ],
                    'required': [
                        'certificate_id',
                        'trusted_certificate',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'certificate_id':
                            (str,),
                        'trusted_certificate':
                            (TrustedCertificate,),
                    },
                    'attribute_map': {
                        'certificate_id': 'certificate_id',
                    },
                    'location_map': {
                        'certificate_id': 'path',
                        'trusted_certificate': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_trusted_cert
            )

    create_bundle = None 
    create_label = None 
    create_trusted_cert = None 
    delete_bundle = None 
    delete_label = None 
    delete_trusted_cert = None 
    get_bundle = None 
    get_label = None 
    get_trusted_cert = None 
    list_bundles = None 
    list_cert_orgs = None 
    list_labels = None 
    list_trusted_certs = None 
    replace_bundle = None 
    replace_trusted_cert = None 
